@@ -11,17 +11,20 @@ use strict;
 use warnings;
 use base qw(Locale::Maketext Exporter);
 use vars qw($VERSION @ISA %Lexicon @EXPORT @EXPORT_OK);
-$VERSION = 0.05;
-@EXPORT = qw(readmo);
+$VERSION = 0.06;
+@EXPORT = qw(read_mo readmo);
 @EXPORT_OK = @EXPORT;
 
 use Encode qw(encode decode FB_DEFAULT);
 use File::Spec::Functions qw(catfile);
 no strict qw(refs);
 
-use vars qw(%Lexicons %ENCODINGS $REREAD_MO $MOFILE);
+use vars qw(%Lexicons %ENCODINGS $REREAD_MO $MOFILE $_AUTO @SYSTEM_LOCALEDIRS);
 $REREAD_MO = 0;
 $MOFILE = "";
+$_AUTO = Locale::Maketext::Gettext::_AUTO::L10N->get_handle;
+@SYSTEM_LOCALEDIRS = qw(/usr/share/locale /usr/lib/locale
+    /usr/local/share/locale /usr/local/lib/locale);
 
 # encoding: Set or retrieve the encoding
 sub encoding {
@@ -75,6 +78,12 @@ sub new {
     # Initialize the MOFILE value of this instance
     $self->{"MOFILE"} = "";
     ${"$class\::MOFILE"} = "" if !defined ${"$class\::MOFILE"};
+    # Find the locale name, for this subclass
+    $self->{"LOCALE"} = $class;
+    $self->{"LOCALE"} =~ s/^.*:://;
+    $self->{"LOCALE"} =~ s/(_)(.*)$/$1 . uc $2/e;
+    # Set the category.  Currently this is always LC_MESSAGES
+    $self->{"CATEGORY"} = "LC_MESSAGES";
     
     $self->init;
     return $self;
@@ -103,7 +112,7 @@ sub bindtextdomain {
 # textdomain: Set the current text domain
 sub textdomain {
     local ($_, %_);
-    my ($self, $class, $DOMAIN, $file);
+    my ($self, $class, $DOMAIN, $LOCALEDIR, $MOfile);
     ($self, $DOMAIN) = @_;
     
     # This is not a static method
@@ -119,37 +128,65 @@ sub textdomain {
     # Set the current domain
     $self->{"DOMAIN"} = $DOMAIN;
     
-    # Find the locale name, for this subclass
-    if (!defined $self->{"LOCALE"}) {
-        $self->{"LOCALE"} = $class;
-        $self->{"LOCALE"} =~ s/^.*:://;
-        $self->{"LOCALE"} =~ s/(_)(.*)$/$1 . uc $2/e;
-    }
-    
     # Clear it
     $self->{"Lexicon"} = {};
     %{"$class\::Lexicon"} = qw();
     $self->{"MOFILE"} = "";
     ${"$class\::MOFILE"} = "";
     
-    # Return if this domain was not binded yet
-    return $DOMAIN if !exists ${$self->{"LOCALEDIRS"}}{$DOMAIN};
+    # The format is "{LOCALEDIR}/{LOCALE}/{CATEGORY}/{DOMAIN}.mo"
+    # Search the system locale directories if the domain was not
+    # registered yet
+    if (!exists ${$self->{"LOCALEDIRS"}}{$DOMAIN}) {
+        undef $MOfile;
+        foreach $LOCALEDIR (@SYSTEM_LOCALEDIRS) {
+            $_ = catfile($LOCALEDIR, $self->{"LOCALE"},
+                $self->{"CATEGORY"}, "$DOMAIN.mo");
+            if (-f $_ && -r $_) {
+                $MOfile = $_;
+                last;
+            }
+        }
+        # Not found at last
+        return $DOMAIN if !defined $MOfile;
+    
+    # This domain was registered
+    } else {
+        $MOfile = catfile(${$self->{"LOCALEDIRS"}}{$DOMAIN},
+            $self->{"LOCALE"}, $self->{"CATEGORY"}, "$DOMAIN.mo");
+    }
     
-    # Import the PO files
-    # The format is "{LOCALEDIR}/{LANG}/LC_MESSAGES/{DOMAIN}.mo"
-    # The category is always LC_MESSAGES.  I'm not planning to change it
-    $file = catfile(${$self->{"LOCALEDIRS"}}{$DOMAIN}, $self->{"LOCALE"}, "LC_MESSAGES", "$DOMAIN.mo");
     # Record it
-    ${"$class\::MOFILE"} = $file;
-    $self->{"MOFILE"} = $file;
+    ${"$class\::MOFILE"} = $MOfile;
+    $self->{"MOFILE"} = $MOfile;
     
     # Read the MO file
-    ($_, %_) = readmo($file);
-    # Keep the current encoding
-    $self->{"ENCODING"} = $_ if !defined $self->{"ENCODING"};
-    $self->{"Lexicon"} = \%_;
-    %{"$class\::Lexicon"} = %_;
-     
+    # Cached
+    if (!exists $ENCODINGS{$MOfile} || !exists $Lexicons{$MOfile}) {
+        my $enc;
+        # Read it
+        %_ = read_mo($MOfile);
+        
+        # Successfully read
+        if (scalar(keys %_) > 0) {
+            # Decode it
+            # Find the encoding of that MO file
+            $_{""} =~ /^Content-Type: text\/plain; charset=(.*)$/im;
+            $enc = $1;
+            # Set the current encoding to the encoding of the MO file
+            $_{$_} = decode($enc, $_{$_}) foreach keys %_;
+        }
+        # Cache them
+        $Lexicons{$MOfile} = \%_;
+        $ENCODINGS{$MOfile} = $enc;
+    }
+    
+    # Respect the existing output encoding setting
+    $self->{"ENCODING"} = $ENCODINGS{$MOfile}
+        if !defined $self->{"ENCODING"};
+    $self->{"Lexicon"} = $Lexicons{$MOfile};
+    %{"$class\::Lexicon"} = %{$Lexicons{$MOfile}};
+    
     return $DOMAIN;
 }
 
@@ -178,18 +215,10 @@ sub maketext {
     #   several objects for a single localization subclass whenever
     #   possible.
     # Maketext uses class lexicon in order to track the inheritance.
-    #   This is not changable.
-    # Also check if DIE_FOR_LOOKUP_FAILURES has the same value as,
-    #   _AUTO, since this affects if the 2 lexicons are identical,
-    #   too.  Maketext caches the compiled result in the same class
-    #   lexicon.  A simply delete is not enough to clear the cache.
-    if (    ${"$class\::MOFILE"} ne $self->{"MOFILE"} ||
-            (!$self->{"DIE_FOR_LOOKUP_FAILURES"} xor
-            (exists ${"$class\::Lexicon"}{"_AUTO"} && ${"$class\::Lexicon"}{"_AUTO"}))) {
+    #   It is hard to change it.
+    if (${"$class\::MOFILE"} ne $self->{"MOFILE"}) {
         ${"$class\::MOFILE"} = $self->{"MOFILE"};
         %{"$class\::Lexicon"} = %{$self->{"Lexicon"}};
-        ${"$class\::Lexicon"}{"_AUTO"} = 1
-            unless $self->{"DIE_FOR_LOOKUP_FAILURES"};
     }
     
     # Decode the _AUTO lexicon first, in order for the _AUTO lexicon
@@ -198,23 +227,52 @@ sub maketext {
         if  !exists ${$self->{"Lexicon"}}{$key} &&
             defined $self->{"KEY_ENCODING"};
     
-    $_ = $self->SUPER::maketext($key, @param);
+    # Process with the _AUTO lexicon for lookup failures
+    if (    !exists ${$self->{"Lexicon"}}{$key}
+            && !$self->{"DIE_FOR_LOOKUP_FAILURES"}) {
+        $_ = $_AUTO->maketext($key, @param);
+    # Process with the ordinary maketext
+    } else {
+        $_ = $self->SUPER::maketext($key, @param);
+    }
+    # Encode with the output encoding
     $_ = encode($self->{"ENCODING"}, $_, $self->{"ENCODE_FAILURE"})
         if defined $self->{"ENCODING"};
     
     return $_;
 }
 
-# readmo: Subroutine to read the MO file
-#         Refer to gettext documentation section 8.3
+# readmo: Deprecated.  Please use read_mo() instead.
 sub readmo {
     local ($_, %_);
-    my ($MOfile, $len, $FH, $content, $tmpl, $enc);
+    my ($MOfile, $enc);
     $MOfile = $_[0];
     
-    # Read before -- return the cached result
-    return ($ENCODINGS{$MOfile}, %{$Lexicons{$MOfile}})
-        if exists $ENCODINGS{$MOfile} && exists $Lexicons{$MOfile};
+    # Deprecated
+    print STDERR "The use of readmo() is deprecated.  Use read_mo() instead.\n";
+    
+    # Read it
+    %_ = read_mo($MOfile);
+    
+    # Successfully read
+    if (scalar(keys %_) > 0) {
+        # Decode it
+        # Find the encoding of that MO file
+        $_{""} =~ /^Content-Type: text\/plain; charset=(.*)$/im;
+        $enc = $1;
+        # Set the output encoding to the encoding of the MO file
+        $_{$_} = decode($enc, $_{$_}) foreach keys %_;
+    }
+    
+    return ($enc, %_);
+}
+
+# read_mo: Subroutine to read and parse the MO file
+#          Refer to gettext documentation section 8.3
+sub read_mo {
+    local ($_, %_);
+    my ($MOfile, $len, $FH, $content, $tmpl);
+    $MOfile = $_[0];
     
     # Avild being stupid
     return unless -f $MOfile && -r $MOfile;
@@ -272,18 +330,7 @@ sub readmo {
         $_{$stro} = $strt;
     }
     
-    # Decode it
-    # Find the encoding of that MO file
-    $_{""} =~ /^Content-Type: text\/plain; charset=(.*)$/im;
-    $enc = $1;
-    # Set the current encoding to the encoding of the MO file
-    $_{$_} = decode($enc, $_{$_}) foreach keys %_;
-    
-    # Cache them
-    $Lexicons{$MOfile} = \%_;
-    $ENCODINGS{$MOfile} = $enc;
-    
-    return ($enc, %_);
+    return %_;
 }
 
 # reload_text: Method to purge the lexicon cache
@@ -313,13 +360,8 @@ sub die_for_lookup_failures {
     # Return the current setting
     return $self->{"DIE_FOR_LOOKUP_FAILURES"} if !defined $is_die;
     
-    # Set to yes
-    if ($is_die) {
-        return ($self->{"DIE_FOR_LOOKUP_FAILURES"} = 1);
-    # Set to no.  GNU gettext never fails.
-    } else {
-        return ($self->{"DIE_FOR_LOOKUP_FAILURES"} = 0);
-    }
+    # Set and return
+    return ($self->{"DIE_FOR_LOOKUP_FAILURES"} = ($is_die? 1: 0));
 }
 
 # encode_failure: What to do if the text is out of your output encoding
@@ -339,6 +381,22 @@ sub encode_failure {
     return $self->{"ENCODE_FAILURE"};
 }
 
+# Public _AUTO lexicon -- process auto lexicon with it, but not the _AUTO
+#   lexicon entry, so that the compiled cache of %Lexicon can be preserved
+#   and reduce the memory-copy and compilation overhead
+package Locale::Maketext::Gettext::_AUTO::L10N;
+use strict;
+use warnings;
+use base qw(Locale::Maketext);
+use vars qw(@ISA %Lexicon);
+
+package Locale::Maketext::Gettext::_AUTO::L10N::i_default;
+use strict;
+use warnings;
+use base qw(Locale::Maketext);
+use vars qw(@ISA %Lexicon);
+%Lexicon = ( "_AUTO" => 1 );
+
 return 1;
 
 __END__
@@ -379,7 +437,7 @@ If you want to have more control to the detail:
 Use Locale::Maketext::Gettext to read and parse the MO file:
 
   use Locale::Maketext::Gettext;
-  ($encoding, %Lexicon) = readmo($MOfile);
+  %Lexicon = read_mo($MOfile);
 
 =head1 DESCRIPTION
 
@@ -406,23 +464,29 @@ Locale::Maketext::Gettext.  That's all. ^_*'
 =item $LH->bindtextdomain(DOMAIN, LOCALEDIR)
 
 Register a text domain with a locale directory.  It is only a
-registration.  Nothing really happens here.  No check is ever made
-whether this C<LOCALEDIR> exists, nor if C<DOMAIN> really sits in
-this C<LOCALEDIR>.  Returns C<LOCALEDIR> itself.  If C<LOCALEDIR> is
-omitted, the registered locale directory of C<DOMAIN> is returned.
+registration.  Nothing really happens.  No check is ever made to
+whether this C<LOCALEDIR> exists, nor if that C<DOMAIN> really sits
+in this C<LOCALEDIR>.  Returns C<LOCALEDIR> itself.  If C<LOCALEDIR>
+is omitted, the registered locale directory of C<DOMAIN> is returned.
 If C<DOMAIN> is not even registered yet, returns C<undef>.  This
 method always success.
 
 =item $LH->textdomain(DOMAIN)
 
-Set the current text domain.  It reads the corresponding MO file
-and replaces the %Lexicon with this new lexicon.  If anything goes
-wrong, for example, MO file not found, unreadable, NFS
-disconnection, etc., it returns immediatly and the your lexicon
-becomes empty.  Returns the C<DOMAIN> itself.  If C<DOMAIN> is
-omitted, the current text domain is returned.  If the current text
-domain is not even set yet, returns C<undef>.  This method always
-success.
+Set the current text domain.  If C<DOMAIN> was not registered with
+C<bindtextdomain> before, search the system locale directories for
+the corresponding MO file.  Then, it reads the MO file and replaces
+your current %Lexicon with this new lexicon.  If anything goes wrong,
+for example, MO file not found, unreadable, NFS disconnection, etc.,
+it returns immediatly and the your lexicon becomes empty.  Returns
+the C<DOMAIN> itself.  If C<DOMAIN> is omitted, the current text
+domain is returned.  If the current text domain is not even set yet,
+returns C<undef>.  This method always success.
+
+The current system locale directory search order is:
+/usr/share/locale, /usr/lib/locale, /usr/local/share/locale,
+/usr/local/lib/locale.  Suggestions for this search order are
+welcome.
 
 =item $LH->language_tag
 
@@ -432,23 +496,9 @@ L<Locale::Maketext(3)|Locale::Maketext/3>.  It is readonly.
 =item $LH->encoding(ENCODING)
 
 Set or retrieve the output encoding.  The default is the same
-encoding as the gettext MO file.  You should not override this method
-in your localization subclasses, as contract to the current
-practice of L<Locale::Maketext(3)|Locale::Maketext/3>.
-
-You should trust the encoding of your gettext MO file.  GNU gettext
-C<msgfmt> checks the illegal characters for you when you compile your
-MO file from your PO file.  The encoding form your MO files are
-always good.
-
-If you try to output to a wrong encoding, part of your text may be
-lost, as C<FB_DEFAULT> does.  If you don't like this C<FB_DEFAULT>,
-change the failure behavior with the method C<encode_failure> below.
-
-If you need the behavior of auto Traditional Chinese/Simplfied
-Chinese conversion, as GNU gettext smartly does, do it yourself with
-the L<Encode::HanExtra(3)|Encode::HanExtra/3>, too.  There may be a
-solution for this in the future, but not now.
+encoding as the gettext MO file.  You should not override this
+method, as contract to the current practice of
+L<Locale::Maketext(3)|Locale::Maketext/3>.
 
 =item $LH->encode_failure(CHECK)
 
@@ -486,23 +536,22 @@ a wrapper that return the text string C<encode>d according to the
 current C<encoding>.  Refer to
 L<Locale::Maketext(3)|Locale::Maketext/3> for its details.
 
-B<NOTICE:> The C<maketext> method is no more static, meaning that
-I<you cannot use MyPackage::L10N::en-E<gt>maketext(...) anymore.>
-That is a sure result, as %Lexicon is imported from the foreign
-source dynamically, but not statically hardcoded in a perl subclass.
-But the documentation of L<Locale::Maketext(3)|Locale::Maketext/3>
-does not say that you can use it as a static method anyway.  Maybe
-you were practicing this before.  You had better check your existing
-code for this.  If you try to invoke it statically, it returns
-C<undef>.
+B<NOTICE:> I<MyPackage::L10N::en-E<gt>maketext(...) is not available
+anymore,> as the C<maketext> method is no more static.  That is a
+sure result, as %Lexicon is imported from foreign sources
+dynamically, but not statically hardcoded in perl sources.  But the
+documentation of L<Locale::Maketext(3)|Locale::Maketext/3> does not
+say that you can use it as a static method anyway.  Maybe you were
+practicing this before.  You had better check your existing code for
+this.  If you try to invoke it statically, it returns C<undef>.
 
 =item $LH->die_for_lookup_failures(SHOULD_I_DIE)
 
 Maketext dies for lookup failures, but GNU gettext never fails.
 By default Lexicon::Maketext::Gettext follows the GNU gettext
-behavior.  But if you are Maketext-styled, or if you want a better
-control over the failures, set this to 1.  Returns the current
-setting.
+behavior.  But if you are Maketext-styled, or if you need a better
+control over the failures (like me :p), set this to 1.  Returns the
+current setting.
 
 =item $LH->reload_text
 
@@ -521,22 +570,35 @@ are running a vital daemon, such as an X display server.
 
 =over
 
-=item ($encoding, %Lexicon) = readmo($MOfile);
+=item %Lexicon = read_mo($MOfile);
 
-Read and parse the MO file, and return a suggested default encoding
-and %Lexicon.  The suggested encoding is the encoding of the MO file
-itself.  This subroutine is called by the C<textdomain> method to
-retrieve the current %Lexicon.  The result is cached, to reduce the
-file I/O and parsing overhead.  This is essential to C<mod_perl>
-where C<textdomain> asks for %Lexicon for each request.  This is
-the same way GNU gettext works.  If you DO need to re-read the
-modified MO file, call the C<reload_text> method above.
+Read and parse the MO file.  Returns the read %Lexicon.  The returned
+lexicon is in its original encoding.
 
-C<readmo()> is exported by default, but you need to C<use
+If you need the meta infomation of your MO file, parse the entry
+C<$Lexicon{""}>.  For example:
+
+  /^Content-Type: text\/plain; charset=(.*)$/im;
+  $encoding = $1;
+
+C<read_mo()> is exported by default, but you need to C<use
 Locale::Maketext::Gettext> in order to use it.  It is not exported
 from your localization class, but from the Locale::Maketext::Gettext
 package.
 
+=item ($encoding, %Lexicon) = readmo($MOfile);
+
+(deprecated) Read and parse the MO file.  Returns a suggested default
+encoding and %Lexicon.  The suggested encoding is the encoding of the
+MO file itself.  The %Lexicon is returned in perl's internal
+encoding.
+
+This method is deprecated and will be removed in the future.  use
+C<read_mo()> instead.  There are far too many meta infomation to be
+returned other than its C<encoding>.  It's not possible to change the
+API for each new requirement.  See C<read_mo()> above for how to
+parse the meta infomation by yourself.
+
 =back
 
 =head1 NOTES
@@ -575,7 +637,7 @@ Locale::Maketext::Gettext also solved the problem of lack of the
 ability to handle the encoding in
 L<Locale::Maketext(3)|Locale::Maketext/3>.  I implement this since
 this is what GNU gettext does.  When %Lexicon is read from MO files
-by C<readmo()>, the encoding tagged in gettext MO files is used to
+by C<read_mo()>, the encoding tagged in gettext MO files is used to
 C<decode> the text into perl's internal encoding.  Then, when
 extracted by C<maketext>, it is C<encode>d by the current
 C<encoding> value.  The C<encoding> can be set at run time, so
@@ -585,6 +647,19 @@ having to restart the application.  This is an improvement to the
 L<Locale::Maketext(3)|Locale::Maketext/3>, and is essential to
 daemons and C<mod_perl> applications.
 
+You should trust the encoding of your gettext MO file.  GNU gettext
+C<msgfmt> checks the illegal characters for you when you compile your
+MO file from your PO file.  The encoding form your MO files are
+always good.  If you try to output to a wrong encoding, part of your
+text may be lost, as C<FB_DEFAULT> does.  If you don't like this
+C<FB_DEFAULT>, change the failure behavior with the method
+C<encode_failure>.
+
+If you need the behavior of auto Traditional Chinese/Simplfied
+Chinese conversion, as GNU gettext smartly does, do it yourself with
+L<Encode::HanExtra(3)|Encode::HanExtra/3>, too.  There may be a
+solution for this in the future, but not now.
+
 C<dgettext> and C<dcgettext> in GNU gettext are not implemented.
 It's not possible to temporarily change the current text domain in
 the current design of Locale::Maketext::Gettext.  Besides, it's
